"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DayWrapper = void 0;
const react_1 = __importDefault(require("react"));
const UI_1 = require("../UI");
const contexts_1 = require("../contexts");
const debounce_1 = require("../helpers/debounce");
const getClassNamesForModifiers_1 = require("../helpers/getClassNamesForModifiers");
const getStyleForModifiers_1 = require("../helpers/getStyleForModifiers");
const selection_1 = require("../selection");
const Day_1 = require("./Day");
const DayDate_1 = require("./DayDate");
/**
 * Provides a `Day` the day state and the html attributes. Developers may use a
 * `Day` component without the need to use hooks.
 *
 * @internal
 */
function DayWrapper(props) {
    const cellRef = react_1.default.useRef(null);
    const { classNames, components, dateLib, dir, formatters: { formatDay }, labels: { labelDay }, locale, mode, modifiersClassNames = {}, modifiersStyles = {}, onDayFocus, onDayBlur, onDayClick, onDayKeyDown, onDayKeyPress, onDayKeyUp, onDayMouseEnter, onDayMouseLeave, onDayPointerEnter, onDayPointerLeave, onDayTouchCancel, onDayTouchEnd, onDayTouchMove, onDayTouchStart, styles = {} } = (0, contexts_1.useProps)();
    const { isInteractive } = (0, contexts_1.useCalendar)();
    const { getModifiers } = (0, contexts_1.useModifiers)();
    const single = (0, selection_1.useSingle)();
    const multi = (0, selection_1.useMulti)();
    const range = (0, selection_1.useRange)();
    const { autoFocusTarget, focused, focus, blur, focusDayBefore, focusDayAfter, focusWeekBefore, focusWeekAfter, focusMonthBefore, focusMonthAfter, focusYearBefore, focusYearAfter, focusStartOfWeek, focusEndOfWeek } = (0, contexts_1.useFocus)();
    const modifiers = getModifiers(props.day);
    const onClick = (e) => {
        if (modifiers.disabled) {
            e.preventDefault();
            e.stopPropagation();
            return;
        }
        switch (mode) {
            case "single":
                single.setSelected(props.day.date, modifiers, e);
                break;
            case "multiple":
                multi.setSelected(props.day.date, modifiers, e);
                break;
            case "range":
                range.setSelected(props.day.date, modifiers, e);
                break;
        }
        if (modifiers.focusable) {
            focus(props.day);
        }
        onDayClick?.(props.day.date, modifiers, e);
    };
    const onFocus = (e) => {
        if (modifiers.disabled) {
            e.preventDefault();
            e.stopPropagation();
            return;
        }
        focus(props.day);
        onDayFocus?.(props.day.date, modifiers, e);
    };
    const onBlur = (e) => {
        blur();
        onDayBlur?.(props.day.date, modifiers, e);
    };
    const onMouseEnter = (e) => {
        onDayMouseEnter?.(props.day.date, modifiers, e);
    };
    const onMouseLeave = (e) => {
        onDayMouseLeave?.(props.day.date, modifiers, e);
    };
    const onPointerEnter = (e) => {
        onDayPointerEnter?.(props.day.date, modifiers, e);
    };
    const onPointerLeave = (e) => {
        onDayPointerLeave?.(props.day.date, modifiers, e);
    };
    const onTouchCancel = (e) => {
        onDayTouchCancel?.(props.day.date, modifiers, e);
    };
    const onTouchEnd = (e) => {
        onDayTouchEnd?.(props.day.date, modifiers, e);
    };
    const onTouchMove = (e) => {
        onDayTouchMove?.(props.day.date, modifiers, e);
    };
    const onTouchStart = (e) => {
        onDayTouchStart?.(props.day.date, modifiers, e);
    };
    const onKeyUp = (e) => {
        onDayKeyUp?.(props.day.date, modifiers, e);
    };
    const onKeyPress = (e) => {
        onDayKeyPress?.(props.day.date, modifiers, e);
    };
    const onKeyDown = (e) => {
        switch (e.key) {
            case "ArrowLeft":
                e.preventDefault();
                e.stopPropagation();
                dir === "rtl" ? focusDayAfter() : focusDayBefore();
                break;
            case "ArrowRight":
                e.preventDefault();
                e.stopPropagation();
                dir === "rtl" ? focusDayBefore() : focusDayAfter();
                break;
            case "ArrowDown":
                e.preventDefault();
                e.stopPropagation();
                focusWeekAfter();
                break;
            case "ArrowUp":
                e.preventDefault();
                e.stopPropagation();
                focusWeekBefore();
                break;
            case " ":
            case "Enter":
                e.preventDefault();
                e.stopPropagation();
                if (mode === "single" && !modifiers.disabled) {
                    single.setSelected(props.day.date, modifiers, e);
                }
                if (mode === "multiple" && !modifiers.disabled) {
                    multi.setSelected(props.day.date, modifiers, e);
                }
                if (mode === "range" && !modifiers.disabled) {
                    range.setSelected(props.day.date, modifiers, e);
                }
                break;
            case "PageUp":
                e.preventDefault();
                e.stopPropagation();
                e.shiftKey ? focusYearBefore() : focusMonthBefore();
                break;
            case "PageDown":
                e.preventDefault();
                e.stopPropagation();
                e.shiftKey ? focusYearAfter() : focusMonthAfter();
                break;
            case "Home":
                e.preventDefault();
                e.stopPropagation();
                focusStartOfWeek();
                break;
            case "End":
                e.preventDefault();
                e.stopPropagation();
                focusEndOfWeek();
                break;
        }
        onDayKeyDown?.(props.day.date, modifiers, e);
    };
    const isAutoFocusTarget = Boolean(autoFocusTarget?.isEqualTo(props.day));
    const isFocused = Boolean(focused?.isEqualTo(props.day));
    const style = (0, getStyleForModifiers_1.getStyleForModifiers)(modifiers, modifiersStyles, styles);
    const classNameForModifiers = (0, getClassNamesForModifiers_1.getClassNamesForModifiers)(modifiers, modifiersClassNames, classNames);
    const className = [classNames[UI_1.UI.Day], ...classNameForModifiers];
    if (isFocused) {
        className.push(classNames[UI_1.DayFlag.focused]);
    }
    const dayRootProps = {
        role: "gridcell",
        className: className.join(" "),
        style,
        tabIndex: isFocused || isAutoFocusTarget ? 0 : -1,
        ["aria-colindex"]: props["aria-colindex"],
        ["aria-label"]: labelDay(props.day.date, modifiers, { locale }, dateLib) ?? undefined,
        ["aria-disabled"]: modifiers.disabled || undefined,
        ["aria-hidden"]: modifiers.hidden || undefined,
        ["aria-selected"]: modifiers.selected || undefined,
        onClick: isInteractive ? onClick : undefined,
        onBlur,
        onFocus,
        onKeyDown,
        onKeyPress: (0, debounce_1.debounce)(onKeyPress, 300),
        onKeyUp,
        onMouseEnter,
        onMouseLeave,
        onPointerEnter,
        onPointerLeave,
        onTouchCancel,
        onTouchEnd,
        onTouchMove,
        onTouchStart,
        ref: cellRef
    };
    react_1.default.useEffect(() => {
        if (!cellRef.current)
            return; // no element to focus
        if (!focused)
            return; // no day to focus
        if (!props.day.isEqualTo(focused))
            return; // not this day`
        if (modifiers.disabled || modifiers.hidden)
            return; // cannot focus
        cellRef.current.focus();
    }, [focused, modifiers.disabled, modifiers.hidden, props.day]);
    const Day = components?.Day ?? Day_1.Day;
    const dayDateRootProps = {
        className: classNames[UI_1.UI.DayDate],
        style: styles[UI_1.UI.DayDate]
    };
    const DayDate = components?.DayDate ?? DayDate_1.DayDate;
    return (react_1.default.createElement(Day, { day: props.day, modifiers: modifiers, rootProps: dayRootProps },
        react_1.default.createElement(DayDate, { day: props.day, modifiers: modifiers, formattedDate: formatDay(props.day.date, { locale }, dateLib), rootProps: dayDateRootProps })));
}
exports.DayWrapper = DayWrapper;
//# sourceMappingURL=DayWrapper.js.map