"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dateMatchModifiers = void 0;
const lib_1 = require("../lib");
const isDateInRange_1 = require("./isDateInRange");
const typeguards_1 = require("./typeguards");
/** Returns true if `value` is a Date type. */
function isDateType(value) {
    return lib_1.dateLib.isDate(value);
}
/** Returns true if `value` is an array of valid dates. */
function isArrayOfDates(value) {
    return Array.isArray(value) && value.every(lib_1.dateLib.isDate);
}
/**
 * Returns whether a day matches against at least one of the given Matchers.
 *
 * ```tsx
 * const day = new Date(2022, 5, 19);
 * const matcher1: DateRange = {
 *  from: new Date(2021, 12, 21),
 *  to: new Date(2021, 12, 30)
 * }
 * const matcher2: DateRange = {
 *  from: new Date(2022, 5, 1),
 *  to: new Date(2022, 5, 23)
 * }
 * const isMatch(day, [matcher1, matcher2]); // true, since day is in the matcher1 range.
 * ```
 *
 * @group Utilities
 */
function dateMatchModifiers(date, matchers, dateUtils = lib_1.dateLib) {
    const matchersArr = !Array.isArray(matchers) ? [matchers] : matchers;
    const { isSameDay, differenceInCalendarDays, isAfter } = dateUtils;
    return matchersArr.some((matcher) => {
        if (typeof matcher === "boolean") {
            return matcher;
        }
        if (isDateType(matcher)) {
            return isSameDay(date, matcher);
        }
        if (isArrayOfDates(matcher)) {
            return matcher.includes(date);
        }
        if ((0, typeguards_1.isDateRange)(matcher)) {
            return (0, isDateInRange_1.isDateInRange)(date, matcher, dateUtils);
        }
        if ((0, typeguards_1.isDayOfWeekType)(matcher)) {
            return matcher.dayOfWeek.includes(date.getDay());
        }
        if ((0, typeguards_1.isDateInterval)(matcher)) {
            const diffBefore = differenceInCalendarDays(matcher.before, date);
            const diffAfter = differenceInCalendarDays(matcher.after, date);
            const isDayBefore = diffBefore > 0;
            const isDayAfter = diffAfter < 0;
            const isClosedInterval = isAfter(matcher.before, matcher.after);
            if (isClosedInterval) {
                return isDayAfter && isDayBefore;
            }
            else {
                return isDayBefore || isDayAfter;
            }
        }
        if ((0, typeguards_1.isDateAfterType)(matcher)) {
            return differenceInCalendarDays(date, matcher.after) > 0;
        }
        if ((0, typeguards_1.isDateBeforeType)(matcher)) {
            return differenceInCalendarDays(matcher.before, date) > 0;
        }
        if (typeof matcher === "function") {
            return matcher(date);
        }
        return false;
    });
}
exports.dateMatchModifiers = dateMatchModifiers;
//# sourceMappingURL=dateMatchModifiers.js.map