import React, { createContext, useContext, useEffect, useState } from "react";
import { DayFlag } from "../UI";
import { getNextFocus } from "../helpers/getNextFocus";
import { useCalendar } from "./useCalendar";
import { useModifiers } from "./useModifiers";
import { useProps } from "./useProps";
const FocusContext = createContext(undefined);
function useFocusContextValue() {
    const { goToDay, isDayDisplayed, isInteractive } = useCalendar();
    const props = useProps();
    const { autoFocus } = props;
    const { dayFlags: internal, selectionStates: selection, getModifiers } = useModifiers();
    const [focused, focus] = useState();
    const [lastFocused, setLastFocused] = useState();
    const [initiallyFocused, setInitiallyFocused] = useState(false);
    const today = internal.today[0];
    let autoFocusTarget;
    const isValidFocusTarget = (day) => {
        return isDayDisplayed(day) && !getModifiers(day)[DayFlag.disabled];
    };
    if (isInteractive) {
        if (focused) {
            autoFocusTarget = focused;
        }
        else if (lastFocused) {
            autoFocusTarget = lastFocused;
        }
        else if (selection.selected[0] &&
            isValidFocusTarget(selection.selected[0])) {
            autoFocusTarget = selection.selected[0];
        }
        else if (today && isValidFocusTarget(today)) {
            autoFocusTarget = today;
        }
        else if (internal.focusable[0]) {
            autoFocusTarget = internal.focusable[0];
        }
    }
    // Focus the focus target when autoFocus is passed in
    useEffect(() => {
        if (!autoFocus)
            return;
        if (!autoFocusTarget)
            return;
        if (!initiallyFocused)
            return;
        // TODO: bug here?
        setInitiallyFocused(true);
        focus(autoFocusTarget);
    }, [autoFocus, autoFocusTarget, focused, initiallyFocused]);
    const blur = () => {
        setLastFocused(focused);
        focus(undefined);
    };
    function moveFocus(moveBy, moveDir) {
        if (!focused)
            return;
        const nextFocus = getNextFocus(moveBy, moveDir, focused, props);
        if (!nextFocus)
            return;
        goToDay(nextFocus);
        focus(nextFocus);
    }
    const contextValue = {
        autoFocusTarget,
        initiallyFocused,
        focus,
        focused,
        setLastFocused,
        blur,
        focusDayAfter: () => moveFocus("day", "after"),
        focusDayBefore: () => moveFocus("day", "before"),
        focusWeekAfter: () => moveFocus("week", "after"),
        focusWeekBefore: () => moveFocus("week", "before"),
        focusMonthBefore: () => moveFocus("month", "before"),
        focusMonthAfter: () => moveFocus("month", "after"),
        focusYearBefore: () => moveFocus("year", "before"),
        focusYearAfter: () => moveFocus("year", "after"),
        focusStartOfWeek: () => moveFocus("startOfWeek", "before"),
        focusEndOfWeek: () => moveFocus("endOfWeek", "after")
    };
    return contextValue;
}
/** @private */
export function FocusContextProvider({ children }) {
    const focusContextValue = useFocusContextValue();
    return (React.createElement(FocusContext.Provider, { value: focusContextValue }, children));
}
/**
 * Share the focused day and the methods to move the focus.
 *
 * Use this hook from the custom components passed via the `components` prop.
 *
 * @group Hooks
 * @see https://react-day-picker.js.org/advanced-guides/custom-components
 */
export function useFocus() {
    const propsContext = useContext(FocusContext);
    if (!propsContext) {
        throw new Error("useFocusContext() must be used within a FocusContextProvider");
    }
    return propsContext;
}
//# sourceMappingURL=useFocus.js.map